@@ -6,6 +6,8 @@ import boto3
 import paramiko
 from botocore.exceptions import ClientError
 from s3transfer import S3Transfer
+import numpy as np
+from tqdm import tqdm
 from . import __version__
 
 logger = logging.getLogger(__name__)
@@ -242,12 +244,12 @@ class DataLake:
             raise e
         return self
 
-    def upload_file(self, local_path, bucket_name, remote_folder_path=None, folder=False):
+    def upload(self, local_path, bucket_name, remote_folder_path=None, folder=False):
         # remote path to have a forward slash in the end test/
         transfer = S3Transfer(self.s3_client)
         try:
             if folder:
-                for file in os.listdir(local_path):
+                for file in tqdm(os.listdir(local_path)):
                     if remote_folder_path is None:
                         transfer.upload_file(filename=f'{local_path}/{file}', bucket=bucket_name,
                                              key=file)
@@ -257,37 +259,49 @@ class DataLake:
 
             else:
                 if remote_folder_path is None:
-                    transfer.upload_file(filename=local_path, bucket=bucket_name,
-                                         key=local_path)
+                    tqdm(transfer.upload_file(filename=local_path, bucket=bucket_name,
+                                              key=local_path))
                 else:
-                    transfer.upload_file(filename=local_path, bucket=bucket_name,
-                                         key=os.path.join(remote_folder_path, os.path.split(local_path)[-1]))
+                    tqdm(transfer.upload_file(filename=local_path, bucket=bucket_name,
+                                              key=os.path.join(remote_folder_path, os.path.split(local_path)[-1])))
         except ClientError as e:
             logger.exception(e)
             raise
 
-    def download_file(self, bucket_name, file_path=None, remote_folder_path=None):
+    def download(self, bucket_name, file_path=None, version=None, remote_folder_path=None):
 
         transfer = S3Transfer(self.s3_client)
         try:
             if remote_folder_path is not None:
 
-                for obj in self.list_files(bucket_name):
-                    if os.path.split(obj)[0] == os.path.normpath(remote_folder_path) and  os.path.split(obj)[-1] != '':
+                for obj in tqdm(self.list_files(bucket_name)):
+                    if os.path.split(obj)[0] == os.path.normpath(remote_folder_path) and os.path.split(obj)[-1] != '':
                         transfer.download_file(bucket=bucket_name, key=obj,
                                                filename=os.path.split(obj)[-1])
                     else:
                         continue
 
             if file_path is not None:
-                transfer.download_file(bucket=bucket_name, key=file_path, filename=os.path.split(file_path)[-1])
+                tqdm(transfer.download_file(bucket=bucket_name, key=file_path, filename=os.path.split(file_path)[-1]))
         except ClientError as e:
             logger.exception(e)
             raise
 
-    def list_files(self, bucket_name):
+    def list_files(self, bucket_name, last_n=None):
 
-        return [obj['Key'] for obj in self.s3_client.list_objects(Bucket=f'{bucket_name}')['Contents']]
+        object_list = [obj['Key'] for obj in self.s3_client.list_objects(Bucket=f'{bucket_name}')['Contents']]
+        object_dates = [obj['LastModified'] for obj in self.s3_client.list_objects(Bucket=f'{bucket_name}')['Contents']]
+
+        if last_n is None:
+            return object_list
+        else:
+            assert last_n < len(object_list), 'last_n cannot be greater than length of object list'
+            return np.array(object_list)[np.argsort(object_dates)[::-1]][:last_n]
+
+    def delete(self, bucket_name, objects):
+        assert isinstance(objects, list), "pass a list to the objects argument even if it's a single file"
+        self.s3_client.delete_objects(Bucket=bucket_name,
+                                      Delete={'Objects': [{'Key': obj} for obj in objects]})
 
     def close_transfer_server(self):
         self.sftp.close()
